@@ -31,16 +31,22 @@
 #define WIN32_NO_STATUS
 #include "windef.h"
 #include "winbase.h"
+#include "winioctl.h"
 #include "winternl.h"
 
 #include "ddk/wdm.h"
+#include "ddk/hidsdi.h"
+#include "ddk/hidport.h"
+#include "ddk/hidclass.h"
 
 DEFINE_GUID(control_class,0xdeadbeef,0x29ef,0x4538,0xa5,0xfd,0xb6,0x95,0x73,0xa3,0x62,0xc0);
 
 #define IOCTL_WINETEST_HID_SET_EXPECT    CTL_CODE(FILE_DEVICE_KEYBOARD, 0x800, METHOD_IN_DIRECT, FILE_ANY_ACCESS)
-#define IOCTL_WINETEST_HID_WAIT_EXPECT   CTL_CODE(FILE_DEVICE_KEYBOARD, 0x801, METHOD_NEITHER, FILE_ANY_ACCESS)
+#define IOCTL_WINETEST_HID_WAIT_EXPECT   CTL_CODE(FILE_DEVICE_KEYBOARD, 0x801, METHOD_IN_DIRECT, FILE_ANY_ACCESS)
 #define IOCTL_WINETEST_HID_SEND_INPUT    CTL_CODE(FILE_DEVICE_KEYBOARD, 0x802, METHOD_IN_DIRECT, FILE_ANY_ACCESS)
 #define IOCTL_WINETEST_HID_SET_CONTEXT   CTL_CODE(FILE_DEVICE_KEYBOARD, 0x803, METHOD_IN_DIRECT, FILE_ANY_ACCESS)
+#define IOCTL_WINETEST_CREATE_DEVICE     CTL_CODE(FILE_DEVICE_KEYBOARD, 0x804, METHOD_IN_DIRECT, FILE_ANY_ACCESS)
+#define IOCTL_WINETEST_REMOVE_DEVICE     CTL_CODE(FILE_DEVICE_KEYBOARD, 0x805, METHOD_IN_DIRECT, FILE_ANY_ACCESS)
 
 struct hid_expect
 {
@@ -55,6 +61,35 @@ struct hid_expect
     BYTE report_buf[128];
 };
 
+#define EXPECT_QUEUE_BUFFER_SIZE (64 * sizeof(struct hid_expect))
+
+struct wait_expect_params
+{
+    BOOL wait_pending;
+};
+
+/* create/remove device */
+#define MAX_HID_DESCRIPTOR_LEN 2048
+
+struct hid_device_desc
+{
+    BOOL is_polled;
+    BOOL use_report_id;
+
+    DWORD report_descriptor_len;
+    char report_descriptor_buf[MAX_HID_DESCRIPTOR_LEN];
+
+    HIDP_CAPS caps;
+    HID_DEVICE_ATTRIBUTES attributes;
+
+    ULONG input_size;
+    struct hid_expect input[64];
+    ULONG expect_size;
+    struct hid_expect expect[64];
+    ULONG context_size;
+    char context[64];
+};
+
 /* kernel/user shared data */
 struct winetest_shared_data
 {
@@ -65,6 +100,81 @@ struct winetest_shared_data
     LONG todo_failures;
 };
 
+static inline const char *debugstr_pnp( ULONG code )
+{
+    switch (code)
+    {
+    case IRP_MN_START_DEVICE: return "IRP_MN_START_DEVICE";
+    case IRP_MN_QUERY_REMOVE_DEVICE: return "IRP_MN_QUERY_REMOVE_DEVICE";
+    case IRP_MN_REMOVE_DEVICE: return "IRP_MN_REMOVE_DEVICE";
+    case IRP_MN_CANCEL_REMOVE_DEVICE: return "IRP_MN_CANCEL_REMOVE_DEVICE";
+    case IRP_MN_STOP_DEVICE: return "IRP_MN_STOP_DEVICE";
+    case IRP_MN_QUERY_STOP_DEVICE: return "IRP_MN_QUERY_STOP_DEVICE";
+    case IRP_MN_CANCEL_STOP_DEVICE: return "IRP_MN_CANCEL_STOP_DEVICE";
+    case IRP_MN_QUERY_DEVICE_RELATIONS: return "IRP_MN_QUERY_DEVICE_RELATIONS";
+    case IRP_MN_QUERY_INTERFACE: return "IRP_MN_QUERY_INTERFACE";
+    case IRP_MN_QUERY_CAPABILITIES: return "IRP_MN_QUERY_CAPABILITIES";
+    case IRP_MN_QUERY_RESOURCES: return "IRP_MN_QUERY_RESOURCES";
+    case IRP_MN_QUERY_RESOURCE_REQUIREMENTS: return "IRP_MN_QUERY_RESOURCE_REQUIREMENTS";
+    case IRP_MN_QUERY_DEVICE_TEXT: return "IRP_MN_QUERY_DEVICE_TEXT";
+    case IRP_MN_FILTER_RESOURCE_REQUIREMENTS: return "IRP_MN_FILTER_RESOURCE_REQUIREMENTS";
+    case IRP_MN_READ_CONFIG: return "IRP_MN_READ_CONFIG";
+    case IRP_MN_WRITE_CONFIG: return "IRP_MN_WRITE_CONFIG";
+    case IRP_MN_EJECT: return "IRP_MN_EJECT";
+    case IRP_MN_SET_LOCK: return "IRP_MN_SET_LOCK";
+    case IRP_MN_QUERY_ID: return "IRP_MN_QUERY_ID";
+    case IRP_MN_QUERY_PNP_DEVICE_STATE: return "IRP_MN_QUERY_PNP_DEVICE_STATE";
+    case IRP_MN_QUERY_BUS_INFORMATION: return "IRP_MN_QUERY_BUS_INFORMATION";
+    case IRP_MN_DEVICE_USAGE_NOTIFICATION: return "IRP_MN_DEVICE_USAGE_NOTIFICATION";
+    case IRP_MN_SURPRISE_REMOVAL: return "IRP_MN_SURPRISE_REMOVAL";
+    case IRP_MN_QUERY_LEGACY_BUS_INFORMATION: return "IRP_MN_QUERY_LEGACY_BUS_INFORMATION";
+    default: return "unknown";
+    }
+}
+
+static inline const char *debugstr_ioctl( ULONG code )
+{
+    switch (code)
+    {
+    case HID_CTL_CODE(0): return "IOCTL_HID_GET_DEVICE_DESCRIPTOR";
+    case HID_CTL_CODE(1): return "IOCTL_HID_GET_REPORT_DESCRIPTOR";
+    case HID_CTL_CODE(2): return "IOCTL_HID_READ_REPORT";
+    case HID_CTL_CODE(3): return "IOCTL_HID_WRITE_REPORT";
+    case HID_CTL_CODE(4): return "IOCTL_HID_GET_STRING";
+    case HID_CTL_CODE(7): return "IOCTL_HID_ACTIVATE_DEVICE";
+    case HID_CTL_CODE(8): return "IOCTL_HID_DEACTIVATE_DEVICE";
+    case HID_CTL_CODE(9): return "IOCTL_HID_GET_DEVICE_ATTRIBUTES";
+    case HID_CTL_CODE(10): return "IOCTL_HID_SEND_IDLE_NOTIFICATION_REQUEST";
+    case HID_OUT_CTL_CODE(102): return "IOCTL_GET_PHYSICAL_DESCRIPTOR";
+    case HID_CTL_CODE(101): return "IOCTL_HID_FLUSH_QUEUE";
+    case HID_CTL_CODE(100): return "IOCTL_HID_GET_COLLECTION_DESCRIPTOR";
+    case HID_BUFFER_CTL_CODE(106): return "IOCTL_HID_GET_COLLECTION_INFORMATION";
+    case HID_OUT_CTL_CODE(100): return "IOCTL_HID_GET_FEATURE";
+    case HID_OUT_CTL_CODE(103): return "IOCTL_HID_GET_HARDWARE_ID";
+    case HID_OUT_CTL_CODE(120): return "IOCTL_HID_GET_INDEXED_STRING";
+    case HID_OUT_CTL_CODE(104): return "IOCTL_HID_GET_INPUT_REPORT";
+    case HID_OUT_CTL_CODE(110): return "IOCTL_HID_GET_MANUFACTURER_STRING";
+    case HID_BUFFER_CTL_CODE(104): return "IOCTL_GET_NUM_DEVICE_INPUT_BUFFERS";
+    case HID_BUFFER_CTL_CODE(102): return "IOCTL_HID_GET_POLL_FREQUENCY_MSEC";
+    case HID_OUT_CTL_CODE(111): return "IOCTL_HID_GET_PRODUCT_STRING";
+    case HID_OUT_CTL_CODE(112): return "IOCTL_HID_GET_SERIALNUMBER_STRING";
+    case HID_IN_CTL_CODE(100): return "IOCTL_HID_SET_FEATURE";
+    case HID_BUFFER_CTL_CODE(105): return "IOCTL_SET_NUM_DEVICE_INPUT_BUFFERS";
+    case HID_IN_CTL_CODE(101): return "IOCTL_HID_SET_OUTPUT_REPORT";
+    case HID_BUFFER_CTL_CODE(103): return "IOCTL_HID_SET_POLL_FREQUENCY_MSEC";
+    case HID_BUFFER_CTL_CODE(100): return "IOCTL_HID_GET_DRIVER_CONFIG";
+    case HID_BUFFER_CTL_CODE(101): return "IOCTL_HID_SET_DRIVER_CONFIG";
+    case HID_OUT_CTL_CODE(121): return "IOCTL_HID_GET_MS_GENRE_DESCRIPTOR";
+    case IOCTL_WINETEST_HID_SET_EXPECT: return "IOCTL_WINETEST_HID_SET_EXPECT";
+    case IOCTL_WINETEST_HID_WAIT_EXPECT: return "IOCTL_WINETEST_HID_WAIT_EXPECT";
+    case IOCTL_WINETEST_HID_SEND_INPUT: return "IOCTL_WINETEST_HID_SEND_INPUT";
+    case IOCTL_WINETEST_HID_SET_CONTEXT: return "IOCTL_WINETEST_HID_SET_CONTEXT";
+    case IOCTL_WINETEST_CREATE_DEVICE: return "IOCTL_WINETEST_CREATE_DEVICE";
+    case IOCTL_WINETEST_REMOVE_DEVICE: return "IOCTL_WINETEST_REMOVE_DEVICE";
+    default: return "unknown";
+    }
+}
+
 #ifndef __WINE_WINE_TEST_H
 
 #if !defined( __WINE_USE_MSVCRT ) || defined( __MINGW32__ )
