@@ -36,6 +36,99 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msdasql);
 
+struct msdasql_prop
+{
+    DBPROPID    property_id;
+    DBPROPFLAGS flags;
+    VARTYPE     vartype;
+
+    LONG value;
+};
+
+static struct msdasql_prop msdasql_init_props[] =
+{
+    { DBPROP_ABORTPRESERVE,                   DBPROPFLAGS_ROWSET | DBPROPFLAGS_DATASOURCEINFO, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_BLOCKINGSTORAGEOBJECTS,          DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_BOOKMARKS,                       DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_BOOKMARKSKIPPED,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_BOOKMARKTYPE,                    DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_I4, 1 },
+    { DBPROP_CANFETCHBACKWARDS,               DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_CANHOLDROWS,                     DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_CANSCROLLBACKWARDS,              DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_COLUMNRESTRICT,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_COMMITPRESERVE,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_DELAYSTORAGEOBJECTS,             DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IMMOBILEROWS,                    DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_LITERALBOOKMARKS,                DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_LITERALIDENTITY,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_MAXOPENROWS,                     DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4, 0 },
+    { DBPROP_MAXPENDINGROWS,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_I4,  0 },
+    { DBPROP_MAXROWS,                         DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_I4, 0 },
+    { DBPROP_NOTIFICATIONPHASES,              DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4, 31 },
+    { DBPROP_OTHERUPDATEDELETE,               DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_OWNINSERT,                       DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_OWNUPDATEDELETE,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_QUICKRESTART ,                   DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_REENTRANTEVENTS,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_REMOVEDELETED,                   DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_REPORTMULTIPLECHANGES,           DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_ROWRESTRICT,                     DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_ROWTHREADMODEL,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4, 2 },
+    { DBPROP_TRANSACTEDOBJECT,                DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_UPDATABILITY,                    DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4, 0 },
+    { DBPROP_STRONGIDENTITY,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IAccessor,                       DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_IColumnsInfo,                    DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_IColumnsRowset,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_IConnectionPointContainer,       DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IRowset,                         DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_IRowsetChange,                   DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IRowsetIdentity,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IRowsetInfo,                     DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_IRowsetLocate,                   DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IRowsetResynch,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IRowsetUpdate,                   DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_ISupportErrorInfo,               DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_ISequentialStream,               DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_NOTIFYCOLUMNSET,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWDELETE,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWFIRSTCHANGE,            DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWINSERT,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWRESYNCH,                DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWSETRELEASE,             DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWSETFETCHPOSITIONCHANGE, DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWUNDOCHANGE,             DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWUNDODELETE,             DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWUNDOINSERT,             DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_NOTIFYROWUPDATE,                 DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4,  3 },
+    { DBPROP_CHANGEINSERTEDROWS,              DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_RETURNPENDINGINSERTS,            DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IConvertType,                    DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_NOTIFICATIONGRANULARITY,         DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_I4, 1 },
+    { DBPROP_IMultipleResults,                DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_ACCESSORDER,                     DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_I4, 1 },
+    { DBPROP_BOOKMARKINFO,                    DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4, 0 },
+    { DBPROP_UNIQUEROWS,                      DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IRowsetFind,                     DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_IRowsetScroll,                   DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_IRowsetRefresh,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_FINDCOMPAREOPS,                  DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ, VT_I4, 27 },
+    { DBPROP_ORDEREDBOOKMARKS,                DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_CLIENTCURSOR,                    DBPROPFLAGS_ROWSET | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_TRUE },
+    { DBPROP_ABORTPRESERVE,                   DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_ACTIVESESSIONS,                  DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_ASYNCTXNCOMMIT,                  DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_AUTH_CACHE_AUTHINFO,             DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_AUTH_ENCRYPT_PASSWORD,           DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_I4, 0 },
+    { DBPROP_AUTH_INTEGRATED,                 DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_I4, 14 },
+    { DBPROP_AUTH_MASK_PASSWORD,              DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_AUTH_PASSWORD,                   DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_AUTH_PERSIST_ENCRYPTED,          DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_AUTH_PERSIST_SENSITIVE_AUTHINFO, DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_AUTH_USERID,                     DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+    { DBPROP_BLOCKINGSTORAGEOBJECTS,          DBPROPFLAGS_DATASOURCEINFO | DBPROPFLAGS_READ | DBPROPFLAGS_WRITE, VT_BOOL, VARIANT_FALSE },
+};
+
 struct msdasql_session
 {
     IUnknown session_iface;
@@ -44,6 +137,7 @@ struct msdasql_session
     ISessionProperties ISessionProperties_iface;
     IDBCreateCommand IDBCreateCommand_iface;
     ITransactionJoin ITransactionJoin_iface;
+    ITransaction ITransaction_iface;
     LONG refs;
 
     IUnknown *datasource;
@@ -81,6 +175,11 @@ static inline struct msdasql_session *impl_from_ITransactionJoin( ITransactionJo
     return CONTAINING_RECORD( iface, struct msdasql_session, ITransactionJoin_iface );
 }
 
+static inline struct msdasql_session *impl_from_ITransaction( ITransaction *iface )
+{
+    return CONTAINING_RECORD( iface, struct msdasql_session, ITransaction_iface );
+}
+
 static HRESULT WINAPI session_QueryInterface(IUnknown *iface, REFIID riid, void **ppv)
 {
     struct msdasql_session *session = impl_from_IUnknown( iface );
@@ -118,6 +217,11 @@ static HRESULT WINAPI session_QueryInterface(IUnknown *iface, REFIID riid, void
         TRACE("(%p)->(ITransactionJoin %p)\n", iface, ppv);
         *ppv = &session->ITransactionJoin_iface;
     }
+    else if(IsEqualGUID(&IID_ITransaction, riid))
+    {
+        TRACE("(%p)->(ITransaction %p)\n", iface, ppv);
+        *ppv = &session->ITransaction_iface;
+    }
     else if(IsEqualGUID(&IID_IBindResource, riid))
     {
         TRACE("(%p)->(IID_IBindResource not support)\n", iface);
@@ -143,7 +247,7 @@ static ULONG WINAPI session_AddRef(IUnknown *iface)
 {
     struct msdasql_session *session = impl_from_IUnknown( iface );
     LONG refs = InterlockedIncrement( &session->refs );
-    TRACE( "%p new refcount %d\n", session, refs );
+    TRACE( "%p new refcount %ld\n", session, refs );
     return refs;
 }
 
@@ -151,7 +255,7 @@ static ULONG WINAPI session_Release(IUnknown *iface)
 {
     struct msdasql_session *session = impl_from_IUnknown( iface );
     LONG refs = InterlockedDecrement( &session->refs );
-    TRACE( "%p new refcount %d\n", session, refs );
+    TRACE( "%p new refcount %ld\n", session, refs );
     if (!refs)
     {
         TRACE( "destroying %p\n", session );
@@ -230,7 +334,7 @@ HRESULT WINAPI openrowset_OpenRowset(IOpenRowset *iface, IUnknown *pUnkOuter, DB
             DBID *index, REFIID riid, ULONG count, DBPROPSET propertysets[], IUnknown **rowset)
 {
     struct msdasql_session *session = impl_from_IOpenRowset( iface );
-    FIXME("%p, %p, %p %p %s, %d %p %p stub\n", session, pUnkOuter, table, index, debugstr_guid(riid),
+    FIXME("%p, %p, %p %p %s, %ld %p %p stub\n", session, pUnkOuter, table, index, debugstr_guid(riid),
             count, propertysets, rowset);
 
     return E_NOTIMPL;
@@ -267,7 +371,7 @@ static HRESULT WINAPI properties_GetProperties(ISessionProperties *iface, ULONG
     const DBPROPIDSET id_sets[], ULONG *count, DBPROPSET **sets)
 {
     struct msdasql_session *session = impl_from_ISessionProperties( iface );
-    FIXME("%p %d %p %p %p\n", session, set_count, id_sets, count, sets);
+    FIXME("%p %lu %p %p %p\n", session, set_count, id_sets, count, sets);
 
     return E_NOTIMPL;
 }
@@ -276,7 +380,7 @@ static HRESULT WINAPI properties_SetProperties(ISessionProperties *iface, ULONG
     DBPROPSET sets[])
 {
     struct msdasql_session *session = impl_from_ISessionProperties( iface );
-    FIXME("%p %d %p\n", session, count, sets);
+    FIXME("%p %lu %p\n", session, count, sets);
 
     return S_OK;
 }
@@ -321,6 +425,9 @@ struct command
     IUnknown *session;
     HDBC hdbc;
     SQLHSTMT hstmt;
+
+    struct msdasql_prop *properties;
+    LONG prop_count;
 };
 
 static inline struct command *impl_from_ICommandText( ICommandText *iface )
@@ -426,7 +533,7 @@ static ULONG WINAPI command_AddRef(ICommandText *iface)
 {
     struct command *command = impl_from_ICommandText( iface );
     LONG refs = InterlockedIncrement( &command->refs );
-    TRACE( "%p new refcount %d\n", command, refs );
+    TRACE( "%p new refcount %ld\n", command, refs );
     return refs;
 }
 
@@ -434,10 +541,12 @@ static ULONG WINAPI command_Release(ICommandText *iface)
 {
     struct command *command = impl_from_ICommandText( iface );
     LONG refs = InterlockedDecrement( &command->refs );
-    TRACE( "%p new refcount %d\n", command, refs );
+    TRACE( "%p new refcount %ld\n", command, refs );
     if (!refs)
     {
         TRACE( "destroying %p\n", command );
+        if (command->properties)
+            heap_free(command->properties);
         if (command->session)
             IUnknown_Release(command->session);
 
@@ -466,6 +575,7 @@ struct msdasql_rowset
     IColumnsRowset IColumnsRowset_iface;
     IUnknown *caller;
     LONG refs;
+    SQLHSTMT hstmt;
 };
 
 static inline struct msdasql_rowset *impl_from_IRowset( IRowset *iface )
@@ -551,7 +661,7 @@ static ULONG WINAPI msdasql_rowset_AddRef(IRowset *iface)
 {
     struct msdasql_rowset *rowset = impl_from_IRowset( iface );
     LONG refs = InterlockedIncrement( &rowset->refs );
-    TRACE( "%p new refcount %d\n", rowset, refs );
+    TRACE( "%p new refcount %ld\n", rowset, refs );
     return refs;
 }
 
@@ -559,11 +669,13 @@ static ULONG WINAPI msdasql_rowset_Release(IRowset *iface)
 {
     struct msdasql_rowset *rowset = impl_from_IRowset( iface );
     LONG refs = InterlockedDecrement( &rowset->refs );
-    TRACE( "%p new refcount %d\n", rowset, refs );
+    TRACE( "%p new refcount %ld\n", rowset, refs );
     if (!refs)
     {
         TRACE( "destroying %p\n", rowset );
 
+        SQLFreeHandle(SQL_HANDLE_STMT, rowset->hstmt);
+
         if (rowset->caller)
             IUnknown_Release(rowset->caller);
 
@@ -576,14 +688,14 @@ static HRESULT WINAPI msdasql_rowset_AddRefRows(IRowset *iface, DBCOUNTITEM coun
         const HROW rows[], DBREFCOUNT ref_counts[], DBROWSTATUS status[])
 {
     struct msdasql_rowset *rowset = impl_from_IRowset( iface );
-    FIXME("%p, %ld, %p, %p, %p\n", rowset, count, rows, ref_counts, status);
+    FIXME("%p, %Id, %p, %p, %p\n", rowset, count, rows, ref_counts, status);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI msdasql_rowset_GetData(IRowset *iface, HROW row, HACCESSOR accessor, void *data)
 {
     struct msdasql_rowset *rowset = impl_from_IRowset( iface );
-    FIXME("%p, %ld, %ld, %p\n", rowset, row, accessor, data);
+    FIXME("%p, %Id, %Id, %p\n", rowset, row, accessor, data);
     return E_NOTIMPL;
 }
 
@@ -591,7 +703,7 @@ static HRESULT WINAPI msdasql_rowset_GetNextRows(IRowset *iface, HCHAPTER reserv
         DBROWCOUNT count, DBCOUNTITEM *obtained, HROW **rows)
 {
     struct msdasql_rowset *rowset = impl_from_IRowset( iface );
-    FIXME("%p, %ld, %ld, %ld, %p, %p\n", rowset, reserved, offset, count, obtained, rows);
+    FIXME("%p, %Id, %Id, %Id, %p, %p\n", rowset, reserved, offset, count, obtained, rows);
     return E_NOTIMPL;
 }
 
@@ -600,14 +712,14 @@ static HRESULT WINAPI msdasql_rowset_ReleaseRows(IRowset *iface, DBCOUNTITEM cou
 {
     struct msdasql_rowset *rowset = impl_from_IRowset( iface );
 
-    FIXME("%p, %ld, %p, %p, %p, %p\n", rowset, count, rows, options, ref_counts, status);
+    FIXME("%p, %Id, %p, %p, %p, %p\n", rowset, count, rows, options, ref_counts, status);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI msdasql_rowset_RestartPosition(IRowset *iface, HCHAPTER reserved)
 {
     struct msdasql_rowset *rowset = impl_from_IRowset( iface );
-    FIXME("%p, %ld\n", rowset, reserved);
+    FIXME("%p, %Id\n", rowset, reserved);
     return E_NOTIMPL;
 }
 
@@ -645,7 +757,7 @@ static HRESULT WINAPI rowset_info_GetProperties(IRowsetInfo *iface, const ULONG
         const DBPROPIDSET propertyidsets[], ULONG *out_count, DBPROPSET **propertysets)
 {
     struct msdasql_rowset *rowset = impl_from_IRowsetInfo( iface );
-    FIXME("%p, %d, %p, %p, %p\n", rowset, count, propertyidsets, out_count, propertysets);
+    FIXME("%p, %lu, %p, %p, %p\n", rowset, count, propertyidsets, out_count, propertysets);
     return E_NOTIMPL;
 }
 
@@ -653,7 +765,7 @@ static HRESULT WINAPI rowset_info_GetReferencedRowset(IRowsetInfo *iface, DBORDI
         REFIID riid, IUnknown **unk)
 {
     struct msdasql_rowset *rowset = impl_from_IRowsetInfo( iface );
-    FIXME("%p, %ld, %s, %p\n", rowset, ordinal, debugstr_guid(riid), unk);
+    FIXME("%p, %Id, %s, %p\n", rowset, ordinal, debugstr_guid(riid), unk);
     return E_NOTIMPL;
 }
 
@@ -713,7 +825,7 @@ static HRESULT WINAPI rowset_colsinfo_MapColumnIDs(IColumnsInfo *iface, DBORDINA
         const DBID *dbids, DBORDINAL *columns)
 {
     struct msdasql_rowset *rowset = rowset_impl_from_IColumnsInfo( iface );
-    FIXME("%p, %lu, %p, %p\n", rowset, column_ids, dbids, columns);
+    FIXME("%p, %Id, %p, %p\n", rowset, column_ids, dbids, columns);
     return E_NOTIMPL;
 }
 
@@ -747,7 +859,7 @@ static ULONG  WINAPI rowset_accessor_Release(IAccessor *iface)
 static HRESULT WINAPI rowset_accessor_AddRefAccessor(IAccessor *iface, HACCESSOR accessor, DBREFCOUNT *count)
 {
     struct msdasql_rowset *rowset = impl_from_IAccessor( iface );
-    FIXME("%p, %lu, %p\n", rowset, accessor, count);
+    FIXME("%p, %Id, %p\n", rowset, accessor, count);
     return E_NOTIMPL;
 }
 
@@ -756,7 +868,7 @@ static HRESULT WINAPI rowset_accessor_CreateAccessor(IAccessor *iface, DBACCESSO
         DBBINDSTATUS status[])
 {
     struct msdasql_rowset *rowset = impl_from_IAccessor( iface );
-    FIXME("%p 0x%08x, %lu, %p, %lu, %p, %p\n", rowset, flags, count, bindings, row_size, accessor, status);
+    FIXME("%p, 0x%08lx, %Id, %p, %Id, %p, %p\n", rowset, flags, count, bindings, row_size, accessor, status);
     return E_NOTIMPL;
 }
 
@@ -764,14 +876,14 @@ static HRESULT WINAPI rowset_accessor_GetBindings(IAccessor *iface, HACCESSOR ac
         DBACCESSORFLAGS *flags, DBCOUNTITEM *count, DBBINDING **bindings)
 {
     struct msdasql_rowset *rowset = impl_from_IAccessor( iface );
-    FIXME("%p %lu, %p, %p, %p\n", rowset, accessor, flags, count, bindings);
+    FIXME("%p, %Id, %p, %p, %p\n", rowset, accessor, flags, count, bindings);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI rowset_accessor_ReleaseAccessor(IAccessor *iface, HACCESSOR accessor, DBREFCOUNT *count)
 {
     struct msdasql_rowset *rowset = impl_from_IAccessor( iface );
-    FIXME("%p, %lu, %p\n", rowset, accessor, count);
+    FIXME("%p, %Id, %p\n", rowset, accessor, count);
     return E_NOTIMPL;
 }
 
@@ -807,15 +919,23 @@ static ULONG WINAPI column_rs_Release(IColumnsRowset *iface)
 static HRESULT WINAPI column_rs_GetAvailableColumns(IColumnsRowset *iface, DBORDINAL *count, DBID **columns)
 {
     struct msdasql_rowset *rowset = impl_from_IColumnsRowset( iface );
-    FIXME("%p, %p, %p\n", rowset, count, columns);
-    return E_NOTIMPL;
+
+    TRACE("%p, %p, %p\n", rowset, count, columns);
+
+    if (!count || !columns)
+        return E_INVALIDARG;
+
+    *count = 0;
+    *columns = NULL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI column_rs_GetColumnsRowset(IColumnsRowset *iface, IUnknown *outer, DBORDINAL count,
         const DBID columns[], REFIID riid, ULONG property_cnt, DBPROPSET property_sets[], IUnknown **unk_rs)
 {
     struct msdasql_rowset *rowset = impl_from_IColumnsRowset( iface );
-    FIXME("(%p)->(%p %ld %p %s %u, %p %p): stub\n", rowset, outer, count, columns, debugstr_guid(riid),
+    FIXME("(%p)->(%p, %Id, %p, %s, %lu, %p, %p): stub\n", rowset, outer, count, columns, debugstr_guid(riid),
           property_cnt, property_sets, unk_rs);
     return E_NOTIMPL;
 }
@@ -834,27 +954,53 @@ static HRESULT WINAPI command_Execute(ICommandText *iface, IUnknown *outer, REFI
 {
     struct command *command = impl_from_ICommandText( iface );
     struct msdasql_rowset *msrowset;
-    HRESULT hr;
+    HRESULT hr = S_OK;
+    RETCODE ret;
+    SQLHSTMT hstmt = command->hstmt;
+    SQLLEN results = -1;
 
-    FIXME("%p, %p, %s, %p %p %p Semi Stub\n", command, outer, debugstr_guid(riid), params, affected, rowset);
+    TRACE("%p, %p, %s, %p %p %p\n", command, outer, debugstr_guid(riid), params, affected, rowset);
 
-    msrowset = heap_alloc(sizeof(*msrowset));
-    if (!msrowset)
-        return E_OUTOFMEMORY;
+    if (!hstmt)
+        SQLAllocHandle(SQL_HANDLE_STMT, command->hdbc, &hstmt);
 
-    msrowset->IRowset_iface.lpVtbl = &msdasql_rowset_vtbl;
-    msrowset->IRowsetInfo_iface.lpVtbl = &rowset_info_vtbl;
-    msrowset->IColumnsInfo_iface.lpVtbl = &rowset_columninfo_vtbll;
-    msrowset->IAccessor_iface.lpVtbl = &accessor_vtbl;
-    msrowset->IColumnsRowset_iface.lpVtbl = &columnrs_rs_vtbl;
-    msrowset->refs = 1;
-    ICommandText_QueryInterface(iface, &IID_IUnknown, (void**)&msrowset->caller);
+    ret = SQLExecDirectW(hstmt, command->query, SQL_NTS);
+    if (ret != SQL_SUCCESS)
+    {
+        dump_sql_diag_records(SQL_HANDLE_STMT, hstmt);
+        return E_FAIL;
+    }
+
+    ret = SQLRowCount(hstmt, &results);
+    if (ret != SQL_SUCCESS)
+        ERR("SQLRowCount failed (%d)\n", ret);
 
     if (affected)
-        *affected = 0; /* FIXME */
+        *affected = results;
 
-    hr = IRowset_QueryInterface(&msrowset->IRowset_iface, riid, (void**)rowset);
-    IRowset_Release(&msrowset->IRowset_iface);
+    *rowset = NULL;
+    if (!wcsnicmp( command->query, L"select ", 7 ))
+    {
+        msrowset = heap_alloc(sizeof(*msrowset));
+        if (!msrowset)
+            return E_OUTOFMEMORY;
+
+        command->hstmt = NULL;
+
+        msrowset->IRowset_iface.lpVtbl = &msdasql_rowset_vtbl;
+        msrowset->IRowsetInfo_iface.lpVtbl = &rowset_info_vtbl;
+        msrowset->IColumnsInfo_iface.lpVtbl = &rowset_columninfo_vtbll;
+        msrowset->IAccessor_iface.lpVtbl = &accessor_vtbl;
+        msrowset->IColumnsRowset_iface.lpVtbl = &columnrs_rs_vtbl;
+        msrowset->refs = 1;
+        ICommandText_QueryInterface(iface, &IID_IUnknown, (void**)&msrowset->caller);
+        msrowset->hstmt = hstmt;
+
+        hr = IRowset_QueryInterface(&msrowset->IRowset_iface, riid, (void**)rowset);
+        IRowset_Release(&msrowset->IRowset_iface);
+    }
+    else
+        SQLFreeStmt(hstmt, SQL_CLOSE);
 
     return hr;
 }
@@ -949,19 +1095,187 @@ static ULONG WINAPI command_prop_Release(ICommandProperties *iface)
     return ICommandText_Release(&command->ICommandText_iface);
 }
 
+static ULONG get_property_count(DWORD flag, struct msdasql_prop *properties, int prop_count)
+{
+    int i, count = 0;
+
+    for(i=0; i < prop_count; i++)
+    {
+        if (properties[i].flags & flag)
+            count++;
+    }
+
+    return count;
+}
+
 static HRESULT WINAPI command_prop_GetProperties(ICommandProperties *iface, ULONG count,
         const DBPROPIDSET propertyidsets[], ULONG *sets_cnt, DBPROPSET **propertyset)
 {
     struct command *command = impl_from_ICommandProperties( iface );
-    FIXME("%p %d %p %p %p\n", command, count, propertyidsets, sets_cnt, propertyset);
-    return E_NOTIMPL;
+    DBPROPSET *propset = NULL;
+    int i, j, k;
+
+    TRACE("%p %ld %p %p %p\n", command, count, propertyidsets, sets_cnt, propertyset);
+
+    /* All Properties */
+    if (count == 0)
+    {
+        int idx;
+        propset = CoTaskMemAlloc(2 * sizeof(DBPROPSET));
+        if (!propset)
+            return E_OUTOFMEMORY;
+
+        propset[0].guidPropertySet = DBPROPSET_ROWSET;
+        propset[0].cProperties = get_property_count(DBPROPFLAGS_ROWSET, command->properties, command->prop_count);
+        propset[0].rgProperties = CoTaskMemAlloc(propset[0].cProperties * sizeof(DBPROP));
+        if (!propset[0].rgProperties)
+        {
+            CoTaskMemFree(propset);
+            return E_OUTOFMEMORY;
+        }
+
+        idx = 0;
+        for (j=0; j < command->prop_count; j++)
+        {
+            if (!(command->properties[j].flags & DBPROPFLAGS_ROWSET))
+                continue;
+            propset[0].rgProperties[idx].dwPropertyID = command->properties[j].property_id;
+
+            V_VT(&propset[0].rgProperties[idx].vValue) = command->properties[j].vartype;
+            if (command->properties[j].vartype == VT_BOOL)
+            {
+                V_BOOL(&propset[0].rgProperties[idx].vValue) = command->properties[j].value;
+            }
+            else if (command->properties[j].vartype == VT_I4)
+            {
+                V_I4(&propset[0].rgProperties[idx].vValue) = command->properties[j].value;
+            }
+            else
+                ERR("Unknown variant type %d\n", command->properties[j].vartype);
+
+            idx++;
+        }
+
+        propset[1].guidPropertySet = DBPROPSET_PROVIDERROWSET;
+        propset[1].cProperties = get_property_count(DBPROPFLAGS_DATASOURCEINFO, command->properties, command->prop_count);
+        propset[1].rgProperties = CoTaskMemAlloc(propset[1].cProperties * sizeof(DBPROP));
+        if (!propset[1].rgProperties)
+        {
+            CoTaskMemFree(propset[0].rgProperties);
+            CoTaskMemFree(propset);
+            return E_OUTOFMEMORY;
+        }
+
+        idx = 0;
+        for (j=0; j < command->prop_count; j++)
+        {
+            if (!(command->properties[j].flags & DBPROPFLAGS_DATASOURCEINFO))
+                continue;
+            propset[1].rgProperties[idx].dwPropertyID = command->properties[j].property_id;
+
+            V_VT(&propset[1].rgProperties[idx].vValue) = command->properties[j].vartype;
+            if (command->properties[j].vartype == VT_BOOL)
+            {
+                V_BOOL(&propset[1].rgProperties[idx].vValue) = command->properties[j].value;
+            }
+            else if (command->properties[j].vartype == VT_I4)
+            {
+                V_I4(&propset[1].rgProperties[idx].vValue) = command->properties[j].value;
+            }
+            else
+                ERR("Unknown variant type %d\n", command->properties[j].vartype);
+
+            idx++;
+        }
+
+        *sets_cnt = 2;
+    }
+    else
+    {
+        propset = CoTaskMemAlloc(count * sizeof(DBPROPSET));
+        if (!propset)
+            return E_OUTOFMEMORY;
+
+        for (i=0; i < count; i++)
+        {
+            TRACE("Property id %d (count %ld, set %s)\n", i, propertyidsets[i].cPropertyIDs,
+                    debugstr_guid(&propertyidsets[i].guidPropertySet));
+
+            propset[i].cProperties = propertyidsets[i].cPropertyIDs;
+            propset[i].rgProperties = CoTaskMemAlloc(propset[i].cProperties * sizeof(DBPROP));
+
+            for (j=0; j < propset[i].cProperties; j++)
+            {
+                propset[i].rgProperties[j].dwPropertyID = propertyidsets[i].rgPropertyIDs[j];
+
+                for(k = 0; k < command->prop_count; k++)
+                {
+                    if (command->properties[k].property_id == propertyidsets[i].rgPropertyIDs[j])
+                    {
+                        V_VT(&propset[i].rgProperties[i].vValue) = command->properties[j].vartype;
+                        if (command->properties[j].vartype == VT_BOOL)
+                        {
+                            V_BOOL(&propset[i].rgProperties[i].vValue) = command->properties[j].value;
+                        }
+                        else if (command->properties[j].vartype == VT_I4)
+                        {
+                            V_I4(&propset[i].rgProperties[i].vValue) = command->properties[j].value;
+                        }
+                        else
+                            ERR("Unknown variant type %d\n", command->properties[j].vartype);
+                        break;
+                    }
+                }
+            }
+        }
+
+        *sets_cnt = count;
+    }
+
+    *propertyset = propset;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI command_prop_SetProperties(ICommandProperties *iface, ULONG count,
         DBPROPSET propertyset[])
 {
     struct command *command = impl_from_ICommandProperties( iface );
-    FIXME("%p %p\n", command, propertyset);
+    int i, j, k;
+
+    TRACE("%p %lu, %p\n", command, count, propertyset);
+
+    for(i=0; i < count; i++)
+    {
+        TRACE("set %s, count %ld\n", debugstr_guid(&propertyset[i].guidPropertySet), propertyset[i].cProperties);
+        for(j=0; j < propertyset[i].cProperties; j++)
+        {
+            for(k=0; k < command->prop_count; k++)
+            {
+                if (command->properties[k].property_id == propertyset[i].rgProperties[j].dwPropertyID)
+                {
+                    TRACE("Found property 0x%08lx\n", command->properties[k].property_id);
+                    if (command->properties[k].flags & DBPROPFLAGS_WRITE)
+                    {
+                        if (command->properties[k].vartype == VT_BOOL)
+                        {
+                            command->properties[k].value = V_BOOL(&propertyset[i].rgProperties[j].vValue);
+                        }
+                        else if (command->properties[k].vartype == VT_I4)
+                        {
+                            command->properties[k].value = V_I4(&propertyset[i].rgProperties[j].vValue);
+                        }
+                        else
+                            ERR("Unknown variant type %d\n", command->properties[j].vartype);
+                    }
+                    else
+                        WARN("Attempting to set Readonly property\n");
+
+                    break;
+                }
+            }
+        }
+    }
     return S_OK;
 }
 
@@ -1004,7 +1318,7 @@ static HRESULT WINAPI colsinfo_MapColumnIDs(IColumnsInfo *iface, DBORDINAL colum
         const DBID *dbids, DBORDINAL *columns)
 {
     struct command *command = impl_from_IColumnsInfo( iface );
-    FIXME("%p, %lu, %p, %p\n", command, column_ids, dbids, columns);
+    FIXME("%p, %Iu, %p, %p\n", command, column_ids, dbids, columns);
     return E_NOTIMPL;
 }
 
@@ -1038,7 +1352,7 @@ static ULONG WINAPI converttype_Release(IConvertType *iface)
 static HRESULT WINAPI converttype_CanConvert(IConvertType *iface, DBTYPE from, DBTYPE to, DBCONVERTFLAGS flags)
 {
     struct command *command = impl_from_IConvertType( iface );
-    FIXME("%p, %u, %d, 0x%08x\n", command, from, to, flags);
+    FIXME("%p, %u, %d, 0x%08lx\n", command, from, to, flags);
     return E_NOTIMPL;
 }
 
@@ -1073,7 +1387,7 @@ static HRESULT WINAPI commandprepare_Prepare(ICommandPrepare *iface, ULONG runs)
     struct command *command = impl_from_ICommandPrepare( iface );
     RETCODE ret;
 
-    TRACE("%p, %u\n", command, runs);
+    TRACE("%p, %lu\n", command, runs);
 
     if (!command->query)
         return DB_E_NOCOMMAND;
@@ -1138,7 +1452,7 @@ static HRESULT WINAPI cmd_with_params_MapParameterNames(ICommandWithParameters *
         LPCWSTR names[], DB_LPARAMS ordinals[])
 {
     struct command *command = impl_from_ICommandWithParameters( iface );
-    FIXME("%p, %ld, %p, %p\n", command, uparams, names, ordinals);
+    FIXME("%p, %Iu, %p, %p\n", command, uparams, names, ordinals);
     return E_NOTIMPL;
 }
 
@@ -1146,7 +1460,7 @@ static HRESULT WINAPI cmd_with_params_SetParameterInfo(ICommandWithParameters *i
         const DB_UPARAMS ordinals[], const DBPARAMBINDINFO bindinfo[])
 {
     struct command *command = impl_from_ICommandWithParameters( iface );
-    FIXME("%p, %ld, %p, %p\n", command, uparams, ordinals, bindinfo);
+    FIXME("%p, %Iu, %p, %p\n", command, uparams, ordinals, bindinfo);
     return E_NOTIMPL;
 }
 
@@ -1187,6 +1501,10 @@ static HRESULT WINAPI createcommand_CreateCommand(IDBCreateCommand *iface, IUnkn
     command->hdbc = session->hdbc;
     command->hstmt = NULL;
 
+    command->prop_count = ARRAY_SIZE(msdasql_init_props);
+    command->properties = heap_alloc(sizeof(msdasql_init_props));
+    memcpy(command->properties, msdasql_init_props, sizeof(msdasql_init_props));
+
     IUnknown_QueryInterface(&session->session_iface, &IID_IUnknown, (void**)&command->session);
 
     hr = ICommandText_QueryInterface(&command->ICommandText_iface, riid, (void**)out);
@@ -1234,7 +1552,7 @@ static HRESULT WINAPI transjoin_JoinTransaction(ITransactionJoin *iface, IUnknow
 {
     struct msdasql_session *session = impl_from_ITransactionJoin( iface );
 
-    FIXME("%p, %p, %d, 0x%08x, %p\n", session, unk, level, flags, options);
+    FIXME("%p, %p, %lu, 0x%08lx, %p\n", session, unk, level, flags, options);
 
     return E_NOTIMPL;
 }
@@ -1248,6 +1566,61 @@ static const ITransactionJoinVtbl transactionjoinVtbl =
     transjoin_JoinTransaction
 };
 
+static HRESULT WINAPI transaction_QueryInterface(ITransaction *iface, REFIID riid, void **out)
+{
+    struct msdasql_session *session = impl_from_ITransaction( iface );
+    return IUnknown_QueryInterface(&session->session_iface, riid, out);
+}
+
+static ULONG WINAPI transaction_AddRef(ITransaction *iface)
+{
+    struct msdasql_session *session = impl_from_ITransaction( iface );
+    return IUnknown_AddRef(&session->session_iface);
+}
+
+static ULONG WINAPI transaction_Release(ITransaction *iface)
+{
+    struct msdasql_session *session = impl_from_ITransaction( iface );
+    return IUnknown_Release(&session->session_iface);
+}
+
+static HRESULT WINAPI transaction_Commit(ITransaction *iface, BOOL retaining, DWORD tc, DWORD rm)
+{
+    struct msdasql_session *session = impl_from_ITransaction( iface );
+
+    FIXME("%p, %d, %ld, %ld\n", session, retaining, tc, rm);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI transaction_Abort(ITransaction *iface, BOID *reason, BOOL retaining, BOOL async)
+{
+    struct msdasql_session *session = impl_from_ITransaction( iface );
+
+    FIXME("%p, %p, %d, %d\n", session, reason, retaining, async);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI transaction_GetTransactionInfo(ITransaction *iface, XACTTRANSINFO *info)
+{
+    struct msdasql_session *session = impl_from_ITransaction( iface );
+
+    FIXME("%p, %p\n", session, info);
+
+    return E_NOTIMPL;
+}
+
+static const ITransactionVtbl transactionVtbl =
+{
+    transaction_QueryInterface,
+    transaction_AddRef,
+    transaction_Release,
+    transaction_Commit,
+    transaction_Abort,
+    transaction_GetTransactionInfo
+};
+
 HRESULT create_db_session(REFIID riid, IUnknown *datasource, HDBC hdbc, void **unk)
 {
     struct msdasql_session *session;
@@ -1263,6 +1636,8 @@ HRESULT create_db_session(REFIID riid, IUnknown *datasource, HDBC hdbc, void **u
     session->ISessionProperties_iface.lpVtbl = &propertiesVtbl;
     session->IDBCreateCommand_iface.lpVtbl = &createcommandVtbl;
     session->ITransactionJoin_iface.lpVtbl = &transactionjoinVtbl;
+    session->ITransaction_iface.lpVtbl = &transactionVtbl;
+
     IUnknown_QueryInterface(datasource, &IID_IUnknown, (void**)&session->datasource);
     session->refs = 1;
     session->hdbc = hdbc;
