@@ -58,6 +58,8 @@ typedef enum {
     EVENTID_SCROLL,
     EVENTID_SELECTIONCHANGE,
     EVENTID_SELECTSTART,
+    EVENTID_STORAGE,
+    EVENTID_STORAGECOMMIT,
     EVENTID_SUBMIT,
     EVENTID_TIMEOUT,
     EVENTID_UNLOAD,
@@ -111,7 +113,8 @@ void dispatch_event(EventTarget*,DOMEvent*) DECLSPEC_HIDDEN;
 HRESULT create_document_event(HTMLDocumentNode*,eventid_t,DOMEvent**) DECLSPEC_HIDDEN;
 HRESULT create_document_event_str(HTMLDocumentNode*,const WCHAR*,IDOMEvent**) DECLSPEC_HIDDEN;
 HRESULT create_event_from_nsevent(nsIDOMEvent*,compat_mode_t,DOMEvent**) DECLSPEC_HIDDEN;
-HRESULT create_message_event(HTMLDocumentNode*,BSTR,DOMEvent**) DECLSPEC_HIDDEN;
+HRESULT create_message_event(HTMLDocumentNode*,VARIANT*,DOMEvent**) DECLSPEC_HIDDEN;
+HRESULT create_storage_event(HTMLDocumentNode*,BSTR,BSTR,BSTR,BOOL,DOMEvent**) DECLSPEC_HIDDEN;
 
 void init_nsevents(HTMLDocumentNode*) DECLSPEC_HIDDEN;
 void release_nsevents(HTMLDocumentNode*) DECLSPEC_HIDDEN;
