@@ -22,6 +22,16 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(wmvcore);
 
+union async_op_data
+{
+    struct
+    {
+        QWORD start;
+        QWORD duration;
+        void *context;
+    } start;
+};
+
 struct async_op
 {
     enum async_op_type
@@ -30,14 +40,20 @@ struct async_op
         ASYNC_OP_STOP,
         ASYNC_OP_CLOSE,
     } type;
-    void *new_context;
+    union async_op_data u;
     struct list entry;
 };
 
+struct sample
+{
+    INSSBuffer *buffer;
+    QWORD pts, duration;
+    DWORD flags;
+    WORD stream;
+};
+
 struct async_reader
 {
-    struct wm_reader reader;
-
     IWMReader IWMReader_iface;
     IWMReaderAdvanced6 IWMReaderAdvanced6_iface;
     IWMReaderAccelerator IWMReaderAccelerator_iface;
@@ -45,11 +61,19 @@ struct async_reader
     IWMReaderStreamClock IWMReaderStreamClock_iface;
     IWMReaderTypeNegotiation IWMReaderTypeNegotiation_iface;
     IReferenceClock IReferenceClock_iface;
+    IUnknown *reader_inner;
+    LONG refcount;
+
+    IWMSyncReader2 *reader;
+    struct wm_reader *wm_reader;
+
+    CRITICAL_SECTION cs;
 
     IWMReaderCallbackAdvanced *callback_advanced;
     IWMReaderCallback *callback;
     void *context;
 
+    REFERENCE_TIME clock_start;
     LARGE_INTEGER clock_frequency;
 
     HANDLE callback_thread;
@@ -71,74 +95,96 @@ static REFERENCE_TIME get_current_time(const struct async_reader *reader)
     return (time.QuadPart * 1000) / reader->clock_frequency.QuadPart * 10000;
 }
 
+static DWORD async_reader_get_wait_timeout(struct async_reader *reader, QWORD pts)
+{
+    REFERENCE_TIME current_time = reader->user_time;
+    DWORD timeout = INFINITE;
+
+    if (!reader->user_clock)
+    {
+        current_time = get_current_time(reader) - reader->clock_start;
+        timeout = (pts - current_time) / 10000;
+    }
+
+    return pts > current_time ? timeout : 0;
+}
+
+static bool async_reader_wait_pts(struct async_reader *reader, QWORD pts)
+{
+    IWMReaderCallbackAdvanced *callback_advanced = reader->callback_advanced;
+    DWORD timeout;
+
+    TRACE("reader %p, pts %I64d.\n", reader, pts);
+
+    if (reader->user_clock && pts > reader->user_time && callback_advanced)
+    {
+        QWORD user_time = reader->user_time;
+        LeaveCriticalSection(&reader->callback_cs);
+        IWMReaderCallbackAdvanced_OnTime(callback_advanced, user_time, reader->context);
+        EnterCriticalSection(&reader->callback_cs);
+    }
+
+    while (reader->running && list_empty(&reader->async_ops))
+    {
+        if (!(timeout = async_reader_get_wait_timeout(reader, pts)))
+            return true;
+        SleepConditionVariableCS(&reader->callback_cv, &reader->callback_cs, timeout);
+    }
+
+    return false;
+}
+
+static void async_reader_deliver_sample(struct async_reader *reader, struct sample *sample)
+{
+    IWMReaderCallbackAdvanced *callback_advanced = reader->callback_advanced;
+    IWMReaderCallback *callback = reader->callback;
+    BOOL read_compressed;
+    HRESULT hr;
+
+    TRACE("reader %p, stream %u, pts %s, duration %s, flags %#lx, buffer %p.\n",
+            reader, sample->stream, debugstr_time(sample->pts), debugstr_time(sample->duration),
+            sample->flags, sample->buffer);
+
+    if (FAILED(hr = IWMSyncReader2_GetReadStreamSamples(reader->reader, sample->stream,
+            &read_compressed)))
+        read_compressed = FALSE;
+
+    LeaveCriticalSection(&reader->callback_cs);
+    if (read_compressed)
+        hr = IWMReaderCallbackAdvanced_OnStreamSample(callback_advanced, sample->stream,
+                sample->pts, sample->duration, sample->flags, sample->buffer, reader->context);
+    else
+        hr = IWMReaderCallback_OnSample(callback, sample->stream - 1, sample->pts, sample->duration,
+                sample->flags, sample->buffer, reader->context);
+    EnterCriticalSection(&reader->callback_cs);
+
+    TRACE("Callback returned %#lx.\n", hr);
+
+    INSSBuffer_Release(sample->buffer);
+}
+
 static void callback_thread_run(struct async_reader *reader)
 {
     IWMReaderCallbackAdvanced *callback_advanced = reader->callback_advanced;
     IWMReaderCallback *callback = reader->callback;
-    REFERENCE_TIME start_time;
-    struct wm_stream *stream;
     static const DWORD zero;
-    QWORD pts, duration;
-    WORD stream_number;
-    INSSBuffer *sample;
     HRESULT hr = S_OK;
-    DWORD flags;
-
-    start_time = get_current_time(reader);
 
     while (reader->running && list_empty(&reader->async_ops))
     {
+        struct sample sample;
+
         LeaveCriticalSection(&reader->callback_cs);
-        hr = wm_reader_get_stream_sample(&reader->reader, callback_advanced, 0, &sample, &pts, &duration, &flags, &stream_number);
+        hr = wm_reader_get_stream_sample(reader->wm_reader, callback_advanced, 0, &sample.buffer,
+                &sample.pts, &sample.duration, &sample.flags, &sample.stream);
         EnterCriticalSection(&reader->callback_cs);
         if (hr != S_OK)
             break;
 
-        stream = wm_reader_get_stream_by_stream_number(&reader->reader, stream_number);
-
-        if (reader->user_clock)
-        {
-            QWORD user_time = reader->user_time;
-
-            if (pts > user_time && callback_advanced)
-            {
-                LeaveCriticalSection(&reader->callback_cs);
-                IWMReaderCallbackAdvanced_OnTime(callback_advanced, user_time, reader->context);
-                EnterCriticalSection(&reader->callback_cs);
-            }
-
-            while (pts > reader->user_time && reader->running && list_empty(&reader->async_ops))
-                SleepConditionVariableCS(&reader->callback_cv, &reader->callback_cs, INFINITE);
-        }
+        if (async_reader_wait_pts(reader, sample.pts))
+            async_reader_deliver_sample(reader, &sample);
         else
-        {
-            while (reader->running && list_empty(&reader->async_ops))
-            {
-                REFERENCE_TIME current_time = get_current_time(reader);
-
-                if (pts <= current_time - start_time)
-                    break;
-
-                SleepConditionVariableCS(&reader->callback_cv, &reader->callback_cs,
-                        (pts - (current_time - start_time)) / 10000);
-            }
-        }
-
-        if (reader->running && list_empty(&reader->async_ops))
-        {
-            LeaveCriticalSection(&reader->callback_cs);
-            if (stream->read_compressed)
-                hr = IWMReaderCallbackAdvanced_OnStreamSample(callback_advanced,
-                        stream_number, pts, duration, flags, sample, reader->context);
-            else
-                hr = IWMReaderCallback_OnSample(callback, stream_number - 1, pts, duration,
-                        flags, sample, reader->context);
-            EnterCriticalSection(&reader->callback_cs);
-
-            TRACE("Callback returned %#lx.\n", hr);
-        }
-
-        INSSBuffer_Release(sample);
+            INSSBuffer_Release(sample.buffer);
     }
 
     if (hr == NS_E_NO_MORE_SAMPLES)
@@ -191,13 +237,17 @@ static DWORD WINAPI async_reader_callback_thread(void *arg)
             struct async_op *op = LIST_ENTRY(entry, struct async_op, entry);
             list_remove(&op->entry);
 
-            if (op->new_context)
-                reader->context = op->new_context;
             hr = list_empty(&reader->async_ops) ? S_OK : E_ABORT;
             switch (op->type)
             {
                 case ASYNC_OP_START:
                 {
+                    reader->context = op->u.start.context;
+                    if (SUCCEEDED(hr))
+                        hr = IWMSyncReader2_SetRange(reader->reader, op->u.start.start, op->u.start.duration);
+                    if (SUCCEEDED(hr))
+                        reader->clock_start = get_current_time(reader);
+
                     LeaveCriticalSection(&reader->callback_cs);
                     IWMReaderCallback_OnStatus(reader->callback, WMT_STARTED, hr,
                             WMT_TYPE_DWORD, (BYTE *)&zero, reader->context);
@@ -291,14 +341,15 @@ error:
     return hr;
 }
 
-static HRESULT async_reader_queue_op(struct async_reader *reader, enum async_op_type type, void *context)
+static HRESULT async_reader_queue_op(struct async_reader *reader, enum async_op_type type, union async_op_data *data)
 {
     struct async_op *op;
 
     if (!(op = calloc(1, sizeof(*op))))
         return E_OUTOFMEMORY;
     op->type = type;
-    op->new_context = context;
+    if (data)
+        op->u = *data;
 
     EnterCriticalSection(&reader->callback_cs);
     list_add_tail(&reader->async_ops, &op->entry);
@@ -317,21 +368,87 @@ static HRESULT WINAPI WMReader_QueryInterface(IWMReader *iface, REFIID iid, void
 {
     struct async_reader *reader = impl_from_IWMReader(iface);
 
-    return IWMProfile3_QueryInterface(&reader->reader.IWMProfile3_iface, iid, out);
+    TRACE("reader %p, iid %s, out %p.\n", reader, debugstr_guid(iid), out);
+
+    if (IsEqualIID(iid, &IID_IUnknown)
+            || IsEqualIID(iid, &IID_IWMReader))
+        *out = &reader->IWMReader_iface;
+    else if (IsEqualIID(iid, &IID_IWMReaderAccelerator))
+        *out = &reader->IWMReaderAccelerator_iface;
+    else if (IsEqualIID(iid, &IID_IWMReaderAdvanced)
+            || IsEqualIID(iid, &IID_IWMReaderAdvanced2)
+            || IsEqualIID(iid, &IID_IWMReaderAdvanced3)
+            || IsEqualIID(iid, &IID_IWMReaderAdvanced4)
+            || IsEqualIID(iid, &IID_IWMReaderAdvanced5)
+            || IsEqualIID(iid, &IID_IWMReaderAdvanced6))
+        *out = &reader->IWMReaderAdvanced6_iface;
+    else if (IsEqualIID(iid, &IID_IWMReaderNetworkConfig)
+            || IsEqualIID(iid, &IID_IWMReaderNetworkConfig2))
+        *out = &reader->IWMReaderNetworkConfig2_iface;
+    else if (IsEqualIID(iid, &IID_IWMReaderStreamClock))
+        *out = &reader->IWMReaderStreamClock_iface;
+    else if (IsEqualIID(iid, &IID_IWMReaderTypeNegotiation))
+        *out = &reader->IWMReaderTypeNegotiation_iface;
+    else if (IsEqualIID(iid, &IID_IWMHeaderInfo)
+            || IsEqualIID(iid, &IID_IWMHeaderInfo2)
+            || IsEqualIID(iid, &IID_IWMHeaderInfo3)
+            || IsEqualIID(iid, &IID_IWMLanguageList)
+            || IsEqualIID(iid, &IID_IWMPacketSize)
+            || IsEqualIID(iid, &IID_IWMPacketSize2)
+            || IsEqualIID(iid, &IID_IWMProfile)
+            || IsEqualIID(iid, &IID_IWMProfile2)
+            || IsEqualIID(iid, &IID_IWMProfile3)
+            || IsEqualIID(iid, &IID_IWMReaderPlaylistBurn)
+            || IsEqualIID(iid, &IID_IWMReaderTimecode))
+        return IUnknown_QueryInterface(reader->reader_inner, iid, out);
+    else if (IsEqualIID(iid, &IID_IReferenceClock))
+        *out = &reader->IReferenceClock_iface;
+    else
+    {
+        WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(iid));
+        return E_NOINTERFACE;
+    }
+
+    IUnknown_AddRef((IUnknown *)*out);
+    return S_OK;
 }
 
 static ULONG WINAPI WMReader_AddRef(IWMReader *iface)
 {
     struct async_reader *reader = impl_from_IWMReader(iface);
-
-    return IWMProfile3_AddRef(&reader->reader.IWMProfile3_iface);
+    ULONG refcount = InterlockedIncrement(&reader->refcount);
+    TRACE("%p increasing refcount to %lu.\n", reader, refcount);
+    return refcount;
 }
 
 static ULONG WINAPI WMReader_Release(IWMReader *iface)
 {
     struct async_reader *reader = impl_from_IWMReader(iface);
+    ULONG refcount = InterlockedDecrement(&reader->refcount);
 
-    return IWMProfile3_Release(&reader->reader.IWMProfile3_iface);
+    TRACE("%p decreasing refcount to %lu.\n", reader, refcount);
+
+    if (!refcount)
+    {
+        EnterCriticalSection(&reader->callback_cs);
+        reader->running = false;
+        LeaveCriticalSection(&reader->callback_cs);
+        WakeConditionVariable(&reader->callback_cv);
+
+        async_reader_close(reader);
+
+        reader->callback_cs.DebugInfo->Spare[0] = 0;
+        DeleteCriticalSection(&reader->callback_cs);
+        reader->cs.DebugInfo->Spare[0] = 0;
+        DeleteCriticalSection(&reader->cs);
+
+        IWMSyncReader2_Close(reader->reader);
+
+        IUnknown_Release(reader->reader_inner);
+        free(reader);
+    }
+
+    return refcount;
 }
 
 static HRESULT WINAPI WMReader_Open(IWMReader *iface, const WCHAR *url,
@@ -343,20 +460,13 @@ static HRESULT WINAPI WMReader_Open(IWMReader *iface, const WCHAR *url,
     TRACE("reader %p, url %s, callback %p, context %p.\n",
             reader, debugstr_w(url), callback, context);
 
-    EnterCriticalSection(&reader->reader.cs);
+    EnterCriticalSection(&reader->cs);
 
-    if (reader->reader.wg_parser)
-    {
-        LeaveCriticalSection(&reader->reader.cs);
-        WARN("Stream is already open; returning E_UNEXPECTED.\n");
-        return E_UNEXPECTED;
-    }
-
-    if (SUCCEEDED(hr = wm_reader_open_file(&reader->reader, url))
+    if (SUCCEEDED(hr = IWMSyncReader2_Open(reader->reader, url))
             && FAILED(hr = async_reader_open(reader, callback, context)))
-        wm_reader_close(&reader->reader);
+        IWMSyncReader2_Close(reader->reader);
 
-    LeaveCriticalSection(&reader->reader.cs);
+    LeaveCriticalSection(&reader->cs);
     return hr;
 }
 
@@ -367,15 +477,15 @@ static HRESULT WINAPI WMReader_Close(IWMReader *iface)
 
     TRACE("reader %p.\n", reader);
 
-    EnterCriticalSection(&reader->reader.cs);
+    EnterCriticalSection(&reader->cs);
 
     if (SUCCEEDED(hr = async_reader_queue_op(reader, ASYNC_OP_CLOSE, NULL)))
     {
         async_reader_close(reader);
-        hr = wm_reader_close(&reader->reader);
+        hr = IWMSyncReader2_Close(reader->reader);
     }
 
-    LeaveCriticalSection(&reader->reader.cs);
+    LeaveCriticalSection(&reader->cs);
 
     return hr;
 }
@@ -386,9 +496,9 @@ static HRESULT WINAPI WMReader_GetOutputCount(IWMReader *iface, DWORD *count)
 
     TRACE("reader %p, count %p.\n", reader, count);
 
-    EnterCriticalSection(&reader->reader.cs);
-    *count = reader->reader.stream_count;
-    LeaveCriticalSection(&reader->reader.cs);
+    EnterCriticalSection(&reader->wm_reader->cs);
+    *count = reader->wm_reader->stream_count;
+    LeaveCriticalSection(&reader->wm_reader->cs);
     return S_OK;
 }
 
@@ -398,7 +508,7 @@ static HRESULT WINAPI WMReader_GetOutputProps(IWMReader *iface, DWORD output, IW
 
     TRACE("reader %p, output %lu, props %p.\n", reader, output, props);
 
-    return wm_reader_get_output_props(&reader->reader, output, props);
+    return wm_reader_get_output_props(reader->wm_reader, output, props);
 }
 
 static HRESULT WINAPI WMReader_SetOutputProps(IWMReader *iface, DWORD output, IWMOutputMediaProps *props)
@@ -407,7 +517,7 @@ static HRESULT WINAPI WMReader_SetOutputProps(IWMReader *iface, DWORD output, IW
 
     TRACE("reader %p, output %lu, props %p.\n", reader, output, props);
 
-    return wm_reader_set_output_props(&reader->reader, output, props);
+    return wm_reader_set_output_props(reader->wm_reader, output, props);
 }
 
 static HRESULT WINAPI WMReader_GetOutputFormatCount(IWMReader *iface, DWORD output, DWORD *count)
@@ -416,7 +526,7 @@ static HRESULT WINAPI WMReader_GetOutputFormatCount(IWMReader *iface, DWORD outp
 
     TRACE("reader %p, output %lu, count %p.\n", reader, output, count);
 
-    return wm_reader_get_output_format_count(&reader->reader, output, count);
+    return wm_reader_get_output_format_count(reader->wm_reader, output, count);
 }
 
 static HRESULT WINAPI WMReader_GetOutputFormat(IWMReader *iface, DWORD output,
@@ -426,12 +536,13 @@ static HRESULT WINAPI WMReader_GetOutputFormat(IWMReader *iface, DWORD output,
 
     TRACE("reader %p, output %lu, index %lu, props %p.\n", reader, output, index, props);
 
-    return wm_reader_get_output_format(&reader->reader, output, index, props);
+    return wm_reader_get_output_format(reader->wm_reader, output, index, props);
 }
 
 static HRESULT WINAPI WMReader_Start(IWMReader *iface,
         QWORD start, QWORD duration, float rate, void *context)
 {
+    union async_op_data data = {.start = {.start = start, .duration = duration, .context = context}};
     struct async_reader *reader = impl_from_IWMReader(iface);
     HRESULT hr;
 
@@ -441,17 +552,14 @@ static HRESULT WINAPI WMReader_Start(IWMReader *iface,
     if (rate != 1.0f)
         FIXME("Ignoring rate %.8e.\n", rate);
 
-    EnterCriticalSection(&reader->reader.cs);
+    EnterCriticalSection(&reader->cs);
 
     if (!reader->callback_thread)
         hr = NS_E_INVALID_REQUEST;
     else
-    {
-        wm_reader_seek(&reader->reader, start, duration);
-        hr = async_reader_queue_op(reader, ASYNC_OP_START, context);
-    }
+        hr = async_reader_queue_op(reader, ASYNC_OP_START, &data);
 
-    LeaveCriticalSection(&reader->reader.cs);
+    LeaveCriticalSection(&reader->cs);
 
     return hr;
 }
@@ -463,14 +571,14 @@ static HRESULT WINAPI WMReader_Stop(IWMReader *iface)
 
     TRACE("reader %p.\n", reader);
 
-    EnterCriticalSection(&reader->reader.cs);
+    EnterCriticalSection(&reader->cs);
 
     if (!reader->callback_thread)
         hr = E_UNEXPECTED;
     else
         hr = async_reader_queue_op(reader, ASYNC_OP_STOP, NULL);
 
-    LeaveCriticalSection(&reader->reader.cs);
+    LeaveCriticalSection(&reader->cs);
 
     return hr;
 }
@@ -511,22 +619,22 @@ static struct async_reader *impl_from_IWMReaderAdvanced6(IWMReaderAdvanced6 *ifa
     return CONTAINING_RECORD(iface, struct async_reader, IWMReaderAdvanced6_iface);
 }
 
-static HRESULT WINAPI WMReaderAdvanced_QueryInterface(IWMReaderAdvanced6 *iface, REFIID riid, void **ppv)
+static HRESULT WINAPI WMReaderAdvanced_QueryInterface(IWMReaderAdvanced6 *iface, REFIID iid, void **out)
 {
-    struct async_reader *This = impl_from_IWMReaderAdvanced6(iface);
-    return IWMReader_QueryInterface(&This->IWMReader_iface, riid, ppv);
+    struct async_reader *reader = impl_from_IWMReaderAdvanced6(iface);
+    return IWMReader_QueryInterface(&reader->IWMReader_iface, iid, out);
 }
 
 static ULONG WINAPI WMReaderAdvanced_AddRef(IWMReaderAdvanced6 *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderAdvanced6(iface);
-    return IWMReader_AddRef(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderAdvanced6(iface);
+    return IWMReader_AddRef(&reader->IWMReader_iface);
 }
 
 static ULONG WINAPI WMReaderAdvanced_Release(IWMReaderAdvanced6 *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderAdvanced6(iface);
-    return IWMReader_Release(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderAdvanced6(iface);
+    return IWMReader_Release(&reader->IWMReader_iface);
 }
 
 static HRESULT WINAPI WMReaderAdvanced_SetUserProvidedClock(IWMReaderAdvanced6 *iface, BOOL user_clock)
@@ -538,6 +646,7 @@ static HRESULT WINAPI WMReaderAdvanced_SetUserProvidedClock(IWMReaderAdvanced6 *
     EnterCriticalSection(&reader->callback_cs);
     reader->user_clock = !!user_clock;
     LeaveCriticalSection(&reader->callback_cs);
+    WakeConditionVariable(&reader->callback_cv);
     return S_OK;
 }
 
@@ -592,7 +701,7 @@ static HRESULT WINAPI WMReaderAdvanced_SetStreamsSelected(IWMReaderAdvanced6 *if
     TRACE("reader %p, count %u, stream_numbers %p, selections %p.\n",
             reader, count, stream_numbers, selections);
 
-    return wm_reader_set_streams_selected(&reader->reader, count, stream_numbers, selections);
+    return wm_reader_set_streams_selected(reader->wm_reader, count, stream_numbers, selections);
 }
 
 static HRESULT WINAPI WMReaderAdvanced_GetStreamSelected(IWMReaderAdvanced6 *iface,
@@ -602,7 +711,7 @@ static HRESULT WINAPI WMReaderAdvanced_GetStreamSelected(IWMReaderAdvanced6 *ifa
 
     TRACE("reader %p, stream_number %u, selection %p.\n", reader, stream_number, selection);
 
-    return wm_reader_get_stream_selection(&reader->reader, stream_number, selection);
+    return wm_reader_get_stream_selection(reader->wm_reader, stream_number, selection);
 }
 
 static HRESULT WINAPI WMReaderAdvanced_SetReceiveSelectionCallbacks(IWMReaderAdvanced6 *iface, BOOL get_callbacks)
@@ -626,7 +735,7 @@ static HRESULT WINAPI WMReaderAdvanced_SetReceiveStreamSamples(IWMReaderAdvanced
 
     TRACE("reader %p, stream_number %u, compressed %d.\n", reader, stream_number, compressed);
 
-    return wm_reader_set_read_compressed(&reader->reader, stream_number, compressed);
+    return IWMSyncReader2_SetReadStreamSamples(reader->reader, stream_number, compressed);
 }
 
 static HRESULT WINAPI WMReaderAdvanced_GetReceiveStreamSamples(IWMReaderAdvanced6 *iface, WORD stream_num,
@@ -644,7 +753,7 @@ static HRESULT WINAPI WMReaderAdvanced_SetAllocateForOutput(IWMReaderAdvanced6 *
 
     TRACE("reader %p, output %lu, allocate %d.\n", reader, output, allocate);
 
-    return wm_reader_set_allocate_for_output(&reader->reader, output, allocate);
+    return wm_reader_set_allocate_for_output(reader->wm_reader, output, allocate);
 }
 
 static HRESULT WINAPI WMReaderAdvanced_GetAllocateForOutput(IWMReaderAdvanced6 *iface, DWORD output_num, BOOL *allocate)
@@ -663,7 +772,7 @@ static HRESULT WINAPI WMReaderAdvanced_SetAllocateForStream(IWMReaderAdvanced6 *
 
     TRACE("reader %p, stream_number %u, allocate %d.\n", reader, stream_number, allocate);
 
-    return wm_reader_set_allocate_for_stream(&reader->reader, stream_number, allocate);
+    return wm_reader_set_allocate_for_stream(reader->wm_reader, stream_number, allocate);
 }
 
 static HRESULT WINAPI WMReaderAdvanced_GetAllocateForStream(IWMReaderAdvanced6 *iface, WORD output_num, BOOL *allocate)
@@ -701,7 +810,7 @@ static HRESULT WINAPI WMReaderAdvanced_GetMaxStreamSampleSize(IWMReaderAdvanced6
 
     TRACE("reader %p, stream_number %u, size %p.\n", reader, stream_number, size);
 
-    return wm_reader_get_max_stream_size(&reader->reader, stream_number, size);
+    return wm_reader_get_max_stream_size(reader->wm_reader, stream_number, size);
 }
 
 static HRESULT WINAPI WMReaderAdvanced_NotifyLateDelivery(IWMReaderAdvanced6 *iface, QWORD lateness)
@@ -821,20 +930,13 @@ static HRESULT WINAPI WMReaderAdvanced2_OpenStream(IWMReaderAdvanced6 *iface,
 
     TRACE("reader %p, stream %p, callback %p, context %p.\n", reader, stream, callback, context);
 
-    EnterCriticalSection(&reader->reader.cs);
+    EnterCriticalSection(&reader->cs);
 
-    if (reader->reader.wg_parser)
-    {
-        LeaveCriticalSection(&reader->reader.cs);
-        WARN("Stream is already open; returning E_UNEXPECTED.\n");
-        return E_UNEXPECTED;
-    }
-
-    if (SUCCEEDED(hr = wm_reader_open_stream(&reader->reader, stream))
+    if (SUCCEEDED(hr = IWMSyncReader2_OpenStream(reader->reader, stream))
             && FAILED(hr = async_reader_open(reader, callback, context)))
-        wm_reader_close(&reader->reader);
+        IWMSyncReader2_Close(reader->reader);
 
-    LeaveCriticalSection(&reader->reader.cs);
+    LeaveCriticalSection(&reader->cs);
     return hr;
 }
 
@@ -1000,22 +1102,22 @@ static struct async_reader *impl_from_IWMReaderAccelerator(IWMReaderAccelerator
     return CONTAINING_RECORD(iface, struct async_reader, IWMReaderAccelerator_iface);
 }
 
-static HRESULT WINAPI reader_accl_QueryInterface(IWMReaderAccelerator *iface, REFIID riid, void **object)
+static HRESULT WINAPI reader_accl_QueryInterface(IWMReaderAccelerator *iface, REFIID iid, void **out)
 {
-    struct async_reader *This = impl_from_IWMReaderAccelerator(iface);
-    return IWMReader_QueryInterface(&This->IWMReader_iface, riid, object);
+    struct async_reader *reader = impl_from_IWMReaderAccelerator(iface);
+    return IWMReader_QueryInterface(&reader->IWMReader_iface, iid, out);
 }
 
 static ULONG WINAPI reader_accl_AddRef(IWMReaderAccelerator *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderAccelerator(iface);
-    return IWMReader_AddRef(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderAccelerator(iface);
+    return IWMReader_AddRef(&reader->IWMReader_iface);
 }
 
 static ULONG WINAPI reader_accl_Release(IWMReaderAccelerator *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderAccelerator(iface);
-    return IWMReader_Release(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderAccelerator(iface);
+    return IWMReader_Release(&reader->IWMReader_iface);
 }
 
 static HRESULT WINAPI reader_accl_GetCodecInterface(IWMReaderAccelerator *iface, DWORD output, REFIID riid, void **codec)
@@ -1049,22 +1151,22 @@ static struct async_reader *impl_from_IWMReaderNetworkConfig2(IWMReaderNetworkCo
     return CONTAINING_RECORD(iface, struct async_reader, IWMReaderNetworkConfig2_iface);
 }
 
-static HRESULT WINAPI networkconfig_QueryInterface(IWMReaderNetworkConfig2 *iface, REFIID riid, void **ppv)
+static HRESULT WINAPI networkconfig_QueryInterface(IWMReaderNetworkConfig2 *iface, REFIID iid, void **out)
 {
-    struct async_reader *This = impl_from_IWMReaderNetworkConfig2(iface);
-    return IWMReader_QueryInterface(&This->IWMReader_iface, riid, ppv);
+    struct async_reader *reader = impl_from_IWMReaderNetworkConfig2(iface);
+    return IWMReader_QueryInterface(&reader->IWMReader_iface, iid, out);
 }
 
 static ULONG WINAPI networkconfig_AddRef(IWMReaderNetworkConfig2 *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderNetworkConfig2(iface);
-    return IWMReader_AddRef(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderNetworkConfig2(iface);
+    return IWMReader_AddRef(&reader->IWMReader_iface);
 }
 
 static ULONG WINAPI networkconfig_Release(IWMReaderNetworkConfig2 *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderNetworkConfig2(iface);
-    return IWMReader_Release(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderNetworkConfig2(iface);
+    return IWMReader_Release(&reader->IWMReader_iface);
 }
 
 static HRESULT WINAPI networkconfig_GetBufferingTime(IWMReaderNetworkConfig2 *iface, QWORD *buffering_time)
@@ -1477,22 +1579,22 @@ static struct async_reader *impl_from_IWMReaderStreamClock(IWMReaderStreamClock
     return CONTAINING_RECORD(iface, struct async_reader, IWMReaderStreamClock_iface);
 }
 
-static HRESULT WINAPI readclock_QueryInterface(IWMReaderStreamClock *iface, REFIID riid, void **ppv)
+static HRESULT WINAPI readclock_QueryInterface(IWMReaderStreamClock *iface, REFIID iid, void **out)
 {
-    struct async_reader *This = impl_from_IWMReaderStreamClock(iface);
-    return IWMReader_QueryInterface(&This->IWMReader_iface, riid, ppv);
+    struct async_reader *reader = impl_from_IWMReaderStreamClock(iface);
+    return IWMReader_QueryInterface(&reader->IWMReader_iface, iid, out);
 }
 
 static ULONG WINAPI readclock_AddRef(IWMReaderStreamClock *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderStreamClock(iface);
-    return IWMReader_AddRef(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderStreamClock(iface);
+    return IWMReader_AddRef(&reader->IWMReader_iface);
 }
 
 static ULONG WINAPI readclock_Release(IWMReaderStreamClock *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderStreamClock(iface);
-    return IWMReader_Release(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderStreamClock(iface);
+    return IWMReader_Release(&reader->IWMReader_iface);
 }
 
 static HRESULT WINAPI readclock_GetTime(IWMReaderStreamClock *iface, QWORD *now)
@@ -1533,22 +1635,22 @@ static struct async_reader *impl_from_IWMReaderTypeNegotiation(IWMReaderTypeNego
     return CONTAINING_RECORD(iface, struct async_reader, IWMReaderTypeNegotiation_iface);
 }
 
-static HRESULT WINAPI negotiation_QueryInterface(IWMReaderTypeNegotiation *iface, REFIID riid, void **ppv)
+static HRESULT WINAPI negotiation_QueryInterface(IWMReaderTypeNegotiation *iface, REFIID iid, void **out)
 {
-    struct async_reader *This = impl_from_IWMReaderTypeNegotiation(iface);
-    return IWMReader_QueryInterface(&This->IWMReader_iface, riid, ppv);
+    struct async_reader *reader = impl_from_IWMReaderTypeNegotiation(iface);
+    return IWMReader_QueryInterface(&reader->IWMReader_iface, iid, out);
 }
 
 static ULONG WINAPI negotiation_AddRef(IWMReaderTypeNegotiation *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderTypeNegotiation(iface);
-    return IWMReader_AddRef(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderTypeNegotiation(iface);
+    return IWMReader_AddRef(&reader->IWMReader_iface);
 }
 
 static ULONG WINAPI negotiation_Release(IWMReaderTypeNegotiation *iface)
 {
-    struct async_reader *This = impl_from_IWMReaderTypeNegotiation(iface);
-    return IWMReader_Release(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IWMReaderTypeNegotiation(iface);
+    return IWMReader_Release(&reader->IWMReader_iface);
 }
 
 static HRESULT WINAPI negotiation_TryOutputProps(IWMReaderTypeNegotiation *iface, DWORD output, IWMOutputMediaProps *props)
@@ -1573,22 +1675,22 @@ static struct async_reader *impl_from_IReferenceClock(IReferenceClock *iface)
     return CONTAINING_RECORD(iface, struct async_reader, IReferenceClock_iface);
 }
 
-static HRESULT WINAPI refclock_QueryInterface(IReferenceClock *iface, REFIID riid, void **ppv)
+static HRESULT WINAPI refclock_QueryInterface(IReferenceClock *iface, REFIID iid, void **out)
 {
-    struct async_reader *This = impl_from_IReferenceClock(iface);
-    return IWMReader_QueryInterface(&This->IWMReader_iface, riid, ppv);
+    struct async_reader *reader = impl_from_IReferenceClock(iface);
+    return IWMReader_QueryInterface(&reader->IWMReader_iface, iid, out);
 }
 
 static ULONG WINAPI refclock_AddRef(IReferenceClock *iface)
 {
-    struct async_reader *This = impl_from_IReferenceClock(iface);
-    return IWMReader_AddRef(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IReferenceClock(iface);
+    return IWMReader_AddRef(&reader->IWMReader_iface);
 }
 
 static ULONG WINAPI refclock_Release(IReferenceClock *iface)
 {
-    struct async_reader *This = impl_from_IReferenceClock(iface);
-    return IWMReader_Release(&This->IWMReader_iface);
+    struct async_reader *reader = impl_from_IReferenceClock(iface);
+    return IWMReader_Release(&reader->IWMReader_iface);
 }
 
 static HRESULT WINAPI refclock_GetTime(IReferenceClock *iface, REFERENCE_TIME *time)
@@ -1640,86 +1742,16 @@ static const IReferenceClockVtbl ReferenceClockVtbl =
     refclock_Unadvise
 };
 
-static struct async_reader *impl_from_wm_reader(struct wm_reader *iface)
-{
-    return CONTAINING_RECORD(iface, struct async_reader, reader);
-}
-
-static void *async_reader_query_interface(struct wm_reader *iface, REFIID iid)
-{
-    struct async_reader *reader = impl_from_wm_reader(iface);
-
-    TRACE("reader %p, iid %s.\n", reader, debugstr_guid(iid));
-
-    if (IsEqualIID(iid, &IID_IReferenceClock))
-        return &reader->IReferenceClock_iface;
-
-    if (IsEqualIID(iid, &IID_IWMReader))
-        return &reader->IWMReader_iface;
-
-    if (IsEqualIID(iid, &IID_IWMReaderAccelerator))
-        return &reader->IWMReaderAccelerator_iface;
-
-    if (IsEqualIID(iid, &IID_IWMReaderAdvanced)
-            || IsEqualIID(iid, &IID_IWMReaderAdvanced2)
-            || IsEqualIID(iid, &IID_IWMReaderAdvanced3)
-            || IsEqualIID(iid, &IID_IWMReaderAdvanced4)
-            || IsEqualIID(iid, &IID_IWMReaderAdvanced5)
-            || IsEqualIID(iid, &IID_IWMReaderAdvanced6))
-        return &reader->IWMReaderAdvanced6_iface;
-
-    if (IsEqualIID(iid, &IID_IWMReaderNetworkConfig)
-            || IsEqualIID(iid, &IID_IWMReaderNetworkConfig2))
-        return &reader->IWMReaderNetworkConfig2_iface;
-
-    if (IsEqualIID(iid, &IID_IWMReaderStreamClock))
-        return &reader->IWMReaderStreamClock_iface;
-
-    if (IsEqualIID(iid, &IID_IWMReaderTypeNegotiation))
-        return &reader->IWMReaderTypeNegotiation_iface;
-
-    return NULL;
-}
-
-static void async_reader_destroy(struct wm_reader *iface)
-{
-    struct async_reader *reader = impl_from_wm_reader(iface);
-
-    TRACE("reader %p.\n", reader);
-
-    EnterCriticalSection(&reader->callback_cs);
-    reader->running = false;
-    LeaveCriticalSection(&reader->callback_cs);
-    WakeConditionVariable(&reader->callback_cv);
-
-    async_reader_close(reader);
-
-    reader->callback_cs.DebugInfo->Spare[0] = 0;
-    DeleteCriticalSection(&reader->callback_cs);
-
-    wm_reader_close(&reader->reader);
-
-    wm_reader_cleanup(&reader->reader);
-    free(reader);
-}
-
-static const struct wm_reader_ops async_reader_ops =
-{
-    .query_interface = async_reader_query_interface,
-    .destroy = async_reader_destroy,
-};
-
 HRESULT WINAPI winegstreamer_create_wm_async_reader(IWMReader **reader)
 {
     struct async_reader *object;
+    HRESULT hr;
 
     TRACE("reader %p.\n", reader);
 
     if (!(object = calloc(1, sizeof(*object))))
         return E_OUTOFMEMORY;
 
-    wm_reader_init(&object->reader, &async_reader_ops);
-
     object->IReferenceClock_iface.lpVtbl = &ReferenceClockVtbl;
     object->IWMReader_iface.lpVtbl = &WMReaderVtbl;
     object->IWMReaderAdvanced6_iface.lpVtbl = &WMReaderAdvanced6Vtbl;
@@ -1727,7 +1759,20 @@ HRESULT WINAPI winegstreamer_create_wm_async_reader(IWMReader **reader)
     object->IWMReaderNetworkConfig2_iface.lpVtbl = &WMReaderNetworkConfig2Vtbl;
     object->IWMReaderStreamClock_iface.lpVtbl = &WMReaderStreamClockVtbl;
     object->IWMReaderTypeNegotiation_iface.lpVtbl = &WMReaderTypeNegotiationVtbl;
+    object->refcount = 1;
 
+    if (FAILED(hr = winegstreamer_create_wm_sync_reader((IUnknown *)&object->IWMReader_iface,
+            (void **)&object->reader_inner)))
+        goto failed;
+
+    if (FAILED(hr = IUnknown_QueryInterface(object->reader_inner, &IID_IWMSyncReader2,
+            (void **)&object->reader)))
+        goto failed;
+    IWMReader_Release(&object->IWMReader_iface);
+    object->wm_reader = wm_reader_from_sync_reader_inner(object->reader_inner);
+
+    InitializeCriticalSection(&object->cs);
+    object->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": async_reader.cs");
     InitializeCriticalSection(&object->callback_cs);
     object->callback_cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": async_reader.callback_cs");
 
@@ -1737,4 +1782,10 @@ HRESULT WINAPI winegstreamer_create_wm_async_reader(IWMReader **reader)
     TRACE("Created async reader %p.\n", object);
     *reader = (IWMReader *)&object->IWMReader_iface;
     return S_OK;
+
+failed:
+    if (object->reader_inner)
+        IUnknown_Release(object->reader_inner);
+    free(object);
+    return hr;
 }
