@@ -1379,7 +1379,7 @@ struct cv_local_varflag
     unsigned short is_param          : 1;
     unsigned short address_taken     : 1;
     unsigned short from_compiler     : 1; /* generated by compiler */
-    unsigned short is_aggregate      : 1; /* splitted in several variables by compiler */
+    unsigned short is_aggregate      : 1; /* split in several variables by compiler */
     unsigned short from_aggregate    : 1; /* marks a temporary from an aggregate */
     unsigned short is_aliased        : 1;
     unsigned short from_alias        : 1;
@@ -2610,6 +2610,18 @@ typedef struct _PDB_FPO_DATA
     unsigned int   flags;
 } PDB_FPO_DATA;
 
+typedef struct _PDB_STRING_TABLE
+{
+    unsigned int   magic;
+    unsigned int   hash_version;
+    unsigned int   length;
+}
+PDB_STRING_TABLE;
+/* This header is followed by:
+ * - a series (of bytes hdr.length) of 0-terminated strings
+ * - a serialized hash table
+ */
+
 #include "poppack.h"
 
 /* ===================================================
